/**
 * Whether to store compiler sdks in the project or
 * globally in the inwasm app folder.
 * This only applies to auto install mode.
 */
export declare type StoreLocation = 'inwasm' | 'project';
export interface IConfig {
    [key: string]: any;
    zig?: {
        /**
         * Settings to use preinstalled Zig sdk.
         *
         * binary   Path to preinstalled zig binary. Use '$PATH',
         *          if you want to use 'zig' from PATH variable.
         */
        binary: string;
    } | {
        /**
         * Settings to use auto installer for Zig (default).
         *
         * version  Zig version to use (default is 'master')
         * store    whether to store Zig sdk on project or inwasm (default is 'project')
         */
        version: string;
        store: StoreLocation;
    };
    emsdk?: {
        /**
         * Settings to use preinstalled emsdk.
         *
         * path     path to emsdk folder
         */
        path: string;
    } | {
        /**
         * Settings to use auto installer for emsdk (default).
         *
         * version  emsdk version to use (default is 'latest')
         * store    whether to store emsdk on project or inwasm (default is 'project')
         */
        version: string;
        store: StoreLocation;
    };
}
export declare const DEFAULT_CONFIG: IConfig;
export declare const APP_ROOT: string;
export declare const PROJECT_ROOT: string;
export declare const CONFIG: IConfig;
export declare const WABT_PATH = "/usr/bin";
export declare const SHELL = "/bin/bash";
export declare const isPosix: boolean;
interface IWabtToolPath {
    'wasm2c': string;
    'wasm-decompile': string;
    'wasm-objdump': string;
    'wasm-strip': string;
    'wat2wasm': string;
    'wasm2wat': string;
    'wasm-interp': string;
    'wasm-opcodecnt': string;
    'wasm-validate': string;
}
export declare const WABT_TOOL: IWabtToolPath;
export {};
