"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WABT_TOOL = exports.isPosix = exports.SHELL = exports.WABT_PATH = exports.CONFIG = exports.PROJECT_ROOT = exports.APP_ROOT = exports.DEFAULT_CONFIG = void 0;
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
// default config
exports.DEFAULT_CONFIG = {
    zig: {
        version: 'master',
        store: 'project'
    },
    emsdk: {
        version: 'latest',
        store: 'project'
    }
};
/**
 * Return inwasm's application path.
 */
function getAppRoot() {
    // FIXME: needs ESM patch
    let folder = __dirname;
    let found = '';
    while (folder !== path.dirname(folder)) {
        if (fs.existsSync(path.join(folder, 'package.json'))) {
            const content = fs.readFileSync(path.join(folder, 'package.json'), { encoding: 'utf-8' });
            if (JSON.parse(content).name === 'inwasm') {
                found = folder;
                break;
            }
        }
        ;
        folder = path.dirname(folder);
    }
    if (!found)
        throw new Error('cannot determine inwasm app root path');
    return found;
}
exports.APP_ROOT = getAppRoot();
/**
 * Return current project root path inwasm was called for.
 */
function getProjectRoot() {
    let folder = process.env.PWD || process.cwd();
    let found = '';
    while (folder !== path.dirname(folder)) {
        if (fs.existsSync(path.join(folder, 'package.json'))) {
            found = folder;
            break;
        }
        ;
        folder = path.dirname(folder);
    }
    if (!found)
        throw new Error('cannot determine project root path');
    return found;
}
exports.PROJECT_ROOT = getProjectRoot();
function loadConfig(filename) {
    // FIXME: needs ESM shim
    return require(filename);
}
function isObj(item) {
    return (item && typeof item === 'object' && !Array.isArray(item));
}
function merge(target, custom) {
    if (isObj(target) && isObj(custom)) {
        for (const key in custom) {
            if (isObj(custom[key])) {
                Object.assign(target, { [key]: {} });
                merge(target[key], custom[key]);
            }
            else {
                Object.assign(target, { [key]: custom[key] });
            }
        }
    }
    return target;
}
/**
 * Create a config object from env vars.
 */
function getEnvOverrides() {
    const envKeys = Object.keys(process.env).filter(el => el.startsWith('INWASM_'));
    const result = {};
    for (let i = 0; i < envKeys.length; ++i) {
        const path = envKeys[i].split('_').slice(1);
        const value = process.env[envKeys[i]];
        let o = result;
        for (let k = 0; k < path.length; ++k) {
            const lower = path[k].toLowerCase();
            if (k === path.length - 1)
                o[lower] = value;
            else
                o[lower] = {};
            o = o[lower];
        }
    }
    return result;
}
/**
 * Loads config in this order overriding blending from previous settings:
 * - DEFAULT_CONFIG
 * - inwasm.config.js (pulled from PROJECT_ROOT)
 * - env overrides
 */
function getConfig() {
    // load defaults
    const final = Object.assign({}, exports.DEFAULT_CONFIG);
    // merge with config from file
    const configFile = path.join(exports.PROJECT_ROOT, 'inwasm.config.js');
    if (fs.existsSync(configFile)) {
        merge(final, loadConfig(configFile));
    }
    // merge with env overrides
    merge(final, getEnvOverrides());
    // force absolute path expansion for zig.binary and emsdk.path
    if (final.zig && final.zig.hasOwnProperty('binary')) {
        const zigPath = final.zig.binary;
        if (path.basename(zigPath) !== zigPath) {
            final.zig.binary = path.resolve(zigPath);
        }
    }
    if (final.emsdk && final.emsdk.hasOwnProperty('path')) {
        final.emsdk.path = path.resolve(final.emsdk.path);
    }
    return final;
}
exports.CONFIG = getConfig();
exports.WABT_PATH = '/usr/bin';
// shell to be executed
exports.SHELL = '/bin/bash';
// simply assume any OS != windows being POSIX compatible
exports.isPosix = process.platform !== 'win32';
// wabt tool path abstraction
function getWabtTool() {
    const p = (name) => `/usr/bin/${name}`;
    return {
        'wasm2c': p('wasm2c'),
        'wasm-decompile': p('wasm-decompile'),
        'wasm-objdump': p('wasm-objdump'),
        'wasm-strip': p('wasm-strip'),
        'wat2wasm': p('wat2wasm'),
        'wasm2wat': p('wasm2wat'),
        'wasm-interp': p('wasm-interp'),
        'wasm-opcodecnt': p('wasm-opcodecnt'),
        'wasm-validate': p('wasm-validate')
    };
}
exports.WABT_TOOL = getWabtTool();
//# sourceMappingURL=config.js.map