"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.emscriptenRun = exports.getClangBinPath = exports.getEmscriptenPath = void 0;
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const cp = __importStar(require("child_process"));
const helper_1 = require("../helper");
const config_1 = require("../config");
/**
 * Get the path to the emscripten SDK.
 * The call evals the config settings and may try to install
 * the SDK as stated from the config.
 */
// FIXME: make it possible to activate a different version once installed
// FIXME: make 'latest' somehow updateable
function getEmscriptenPath() {
    const emsdkConf = config_1.CONFIG.emsdk;
    if (emsdkConf && emsdkConf.hasOwnProperty('path')) {
        // from preinstalled
        return emsdkConf.path;
    }
    // from autoinstalled
    const basePath = path.join(emsdkConf.store === 'inwasm' ? config_1.APP_ROOT : config_1.PROJECT_ROOT, 'inwasm-sdks', 'emsdk');
    if (fs.existsSync(basePath) && fs.existsSync(path.join(basePath, 'emsdk.py'))) {
        return basePath;
    }
    (0, helper_1.rmFolder)(basePath);
    // install
    fs.mkdirSync(path.dirname(basePath), { recursive: true });
    console.log(`\n[emscripten.checkout] Cloning emscripten...`);
    cp.execSync(`git clone https://github.com/emscripten-core/emsdk.git ${basePath}`, { shell: config_1.SHELL, stdio: 'inherit' });
    console.log(`\n[emscripten.version] Activate version: "${emsdkConf.version}"`);
    cp.execSync(`${basePath}/emsdk install ${emsdkConf.version} && ${basePath}/emsdk activate ${emsdkConf.version}`, { shell: config_1.SHELL, stdio: 'inherit' });
    return basePath;
}
exports.getEmscriptenPath = getEmscriptenPath;
/**
 * Return the bin/ path of the embedded clang version.
 */
function getClangBinPath() {
    return path.join(getEmscriptenPath(), 'upstream', 'bin');
}
exports.getClangBinPath = getClangBinPath;
/**
 * Run the given command in the emscripten env.
 *
 * Commands like `emcc` can be used directly (implicitly added to PATH by env script).
 * To run executables of the embedded clang version, use `getClangBinPath()` to get a
 * hold of clang's bin/ folder.
 */
function emscriptenRun(cmd) {
    const sdkPath = getEmscriptenPath();
    console.log(`\n[emscripten.run] ${cmd}`);
    const sdk = config_1.isPosix
        ? `source ${path.join(sdkPath, 'emsdk_env.sh')} > /dev/null 2>&1`
        : `${path.join(sdkPath, 'emsdk_env.bat')} >nul 2>&1`;
    cp.execSync(`${sdk} && ${cmd}`, { shell: config_1.SHELL, stdio: 'inherit' });
}
exports.emscriptenRun = emscriptenRun;
//# sourceMappingURL=emscripten.js.map