"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getZigBinary = void 0;
const config_1 = require("../config");
const cp = __importStar(require("child_process"));
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const helper_1 = require("../helper");
const crypto_1 = require("crypto");
function sha256(content) {
    return (0, crypto_1.createHash)('sha256').update(content).digest('hex');
}
// map nodejs arch|platform to Zig naming scheme
const ARCH_MAP = {
    'arm': 'armv7a',
    'arm64': 'aarch64',
    'ia32': 'i386',
    'mips': '#',
    'mipsel': '#',
    'ppc': '#',
    'ppc64': '#',
    's390': '#',
    's390x': '#',
    'x64': 'x86_64'
};
const PLATFORM_MAP = {
    'aix': '#',
    'darwin': 'macos',
    'freebsd': 'freebsd',
    'linux': 'linux',
    'openbsd': '#',
    'sunos': '#',
    'win32': 'windows'
};
function getDownloadInfo() {
    const data = cp.execSync('curl -s https://ziglang.org/download/index.json', { encoding: 'utf-8' });
    return JSON.parse(data);
}
function getUsableVersions() {
    const arch = process.arch;
    const platform = process.platform;
    const zigPlatform = `${ARCH_MAP[arch]}-${PLATFORM_MAP[platform]}`;
    console.log(`[zig.run] ARCH: ${arch}, PLATFORM: ${platform} --> ${zigPlatform}`);
    const downloadInfo = getDownloadInfo();
    const versions = {};
    for (const version of Object.keys(downloadInfo)) {
        if (downloadInfo[version][zigPlatform]) {
            versions[version] = downloadInfo[version][zigPlatform];
        }
    }
    console.log(`[zig.run] Usable Zig versions: [${Object.keys(versions).join(', ')}]`);
    return versions;
}
function downloadAndUnpack(basePath, version, versionString) {
    const tarball = path.join(basePath, config_1.isPosix ? 'sdk.xz' : 'sdk.zip');
    console.log(`[zig.run] Installing Zig "${versionString}"...`);
    cp.execSync(`curl --progress-bar -o ${tarball} ${version.tarball}`, { shell: config_1.SHELL, stdio: 'inherit' });
    if (sha256(fs.readFileSync(tarball)) !== version.shasum)
        throw new Error('download error - shasum does not match');
    if (config_1.isPosix) {
        cp.execSync(`tar -xf ${tarball} -C ${basePath}`);
    }
    else {
        cp.execSync(`${path.join(config_1.APP_ROOT, 'exe', 'unzip.exe')} "${tarball}" -d "${basePath}"`, { stdio: 'ignore' });
    }
    fs.unlinkSync(tarball);
    const subfolder = fs.readdirSync(basePath)[0];
    if (config_1.isPosix) {
        fs.symlinkSync(path.join(basePath, subfolder, 'zig'), path.join(basePath, 'zig'));
    }
    else {
        fs.symlinkSync(path.join(basePath, subfolder), path.join(basePath, 'current'));
    }
    console.log(`[zig.run] Finished.\n`);
}
function localZigBinary(basePath) {
    return config_1.isPosix ? path.join(basePath, 'zig') : path.join(basePath, 'current', 'zig.exe');
}
/**
 * Get the path to the zig binary.
 * The call evals the config settings and may try to install
 * the Zig sdk as stated from the config.
 */
// FIXME: windows support, test on macos
function getZigBinary() {
    const zigConf = config_1.CONFIG.zig;
    if (zigConf && zigConf.hasOwnProperty('binary')) {
        // from preinstalled
        const zigPath = zigConf.binary;
        return zigPath === '$PATH' ? 'zig' : zigPath;
    }
    // requested version
    const versions = getUsableVersions();
    console.log(`[zig.run] Requested version: "${zigConf.version}"`);
    const version = versions[zigConf.version];
    if (!version)
        throw new Error(`zig version ${zigConf.version} not found for your system`);
    // from autoinstalled
    const basePath = path.join(zigConf.store === 'inwasm' ? config_1.APP_ROOT : config_1.PROJECT_ROOT, 'inwasm-sdks', 'zig');
    const localZig = localZigBinary(basePath);
    if (fs.existsSync(basePath) && fs.existsSync(localZig)) {
        const installed = cp.execSync(`${localZig} version`, { encoding: 'utf-8' }).trim();
        if (version.tarball.indexOf(installed) !== -1) {
            return localZig;
        }
        console.log(`[zig.run] Resolving version change "${installed}" --> "${zigConf.version}"`);
    }
    (0, helper_1.rmFolder)(basePath);
    // install
    fs.mkdirSync(basePath, { recursive: true });
    downloadAndUnpack(basePath, version, zigConf.version);
    if (fs.existsSync(localZig)) {
        return localZig;
    }
    throw new Error('cannot find zig binary');
}
exports.getZigBinary = getZigBinary;
//# sourceMappingURL=zig.js.map